//////////////////////////////////////////////////////////////////////
// ANSIBUF                                                          //
//////////////////////////////////////////////////////////////////////
// AnsiBuffer is derived from AnsiOutput.  It takes the ANSI        //
// commands and manipulates a screen buffer.  The buffer is an      //
// array of unsigned shorts (low byte is character, high byte is    //
// the color attribute).                                            //
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <cstring.h>
#include <srcomm3/types.h>
#include <srcomm3/AnsiBuf.h>

AnsiBuffer::AnsiBuffer( int width, int height )
    :width_(width), height_(height)
{
    x_ = 0;
	y_ = 0;
    attr_ = 0x07;

	buffer_ = new unsigned short[height_*width_];
	for( int i = 0; i < height_*width_; i++ )
		buffer_[i] = 0x0700;
}

AnsiBuffer::~AnsiBuffer()
{
    delete [] buffer_;
}

void AnsiBuffer::gotoxy( int x, int y ) { x_ = x-1; y_ = y-1; }
void AnsiBuffer::setattr( int a ) { attr_ = a; }
int AnsiBuffer::whereX() { return 1+x_; }
int AnsiBuffer::whereY() { return 1+y_; }
void AnsiBuffer::beep() {}
int AnsiBuffer::width() { return 1+width_; }
int AnsiBuffer::height() { return 1+height_; }
int AnsiBuffer::currentcolor() { return attr_; }

void AnsiBuffer::clreol()
{
	if( y_ < height_ && y_ >= 0 )
		for( int i = max(x_,0); i < width_; i++ )
			buffer_[ y_*width_ + x_ ] = ( attr_ << 8 ) | ' ';
}

void AnsiBuffer::clrscr( int y1, int y2 )
{
	for( int y = max(y1-1,0); y <= min(y2-1,height_-1); y++ )
	{
		for( int x = 0; x < width_; x++ )
			buffer_[ y*width_ + x_ ] = ( attr_ << 8 ) | ' ';
	}
}

void AnsiBuffer::putchar( char ch )
{
	if( ch == 13 ) { return; }
	if( ch == 10 ) { y_++; x_ = 0; return; }
	unsigned char c = ch;
	if( y_ >= 0 && y_ < height_ && x_ >= 0 && x_ < width_ )
		buffer_[ y_*width_ + x_ ] = ( attr_ << 8 ) | c;
	x_++;
	if( x_ >= 80 )
	{
		x_ = 0;
		if( y_ < height_ ) y_++;
	}
}

