#ifndef _cstring_h
#define _cstring_h

/////////////////////////
// Amit's String class //
/////////////////////////////////////////////////////////////////////
// This is an approximation of the C++ ANSI Standard String Class. //
// All functions are not implemented, and some have different      //
// names.  However, if you are using BC++ 4.0, you _should_ be     //
// able to substitute their <cstring.h> for this one without any   //
// problems.  (I haven't tried it yet.)                            //
/////////////////////////////////////////////////////////////////////

#ifndef CC
#define CC
#endif

#include <_defs.h>
// Try to figure out whether it's BC++ 4.0
#ifdef _USERENTRY
// rename to avoid library conflicts
#define string astring
#endif

const int NPOS = -1;

class string
{
	int len, size;
	char* s;

	string& CC _copy( const string& x );
	string& CC _copy( const char* a );
	string& CC _kill();
	string& CC _resize( int newlen );

  public:
	// Constructors from empty, a character / n characters, a C string,
	// or another C++ string
	CC string();
	CC string( char c, int len = 1 );
	CC string( const char *a );
	CC string( const string& x );

	// Destructor
	~string();

	// Assignment
	string& CC operator = ( const string& x );
	string& CC operator = ( const char* a );

	// Various forms of string mutation

	// Add one character
	string& CC operator += ( char c );
	// Insert a string at the beginning of this one
	string& CC prepend( const string& x );
	string& CC prepend( const char* a );

	// Insert a string at the end of this one
	string& CC operator += ( const string& s );
	string& CC operator += ( const char* s );
	string& CC append( const string& x ) { return (*this) += x; }
	string& CC append( const char* a )   { return (*this) += a; }

	// Return a substring of the string from `start' to `start'+`len'-1
	string CC copy( int start, int len = -1 ) const;
	// Remove a portion of `len' characters starting at `start'
	string& CC remove( int start, int len = 1 );
	// Insert another string at position `pos'
	string& CC insert( int pos, string x );
	// Replace a `len' characters of this string at `pos' with `x'
	string& CC replace( int pos, int len, string x );

	// Return a C-format string (NULL terminated)
	const char * CC c_str() const { return s; }

	// Return a reference to a single character in the string
	char& CC operator () ( int index );

	// Return a substring
	string CC operator () ( int start, int len ) { return copy( start, len ); }

	// Return a character of the string ( this is faster than op() )
	char CC operator [] ( int index ) const;

	// Various comparison operators
	friend int CC operator == ( const string& a, const string& b );
	friend int CC operator != ( const string& a, const string& b );
	friend int CC operator == ( const string& a, const char* b );
	friend int CC operator != ( const string& a, const char* b );
	friend int CC operator < ( const string& a, const string& b );
	friend int CC operator > ( const string& a, const string& b );
	friend int CC operator <= ( const string& a, const string& b );
	friend int CC operator >= ( const string& a, const string& b );

	// String concatenation
	friend string CC operator + ( string a, string b );

	// The length of the string
	int CC length() const { return len; }
	friend int CC length( const string& a ) { return a.len; }

    // Substring searches (for ANSI C++ String compatibility)
    // Be sure to link in strutl.cpp to get these
    int find( const string& s, int start = 0 );
    int contains( const char* s );
    int contains( const string& s );
};

#endif
