//////////////////////////////////////////////////////////////////////
// LONGINT                                                          //
//////////////////////////////////////////////////////////////////////
// I use this for a 'safer' long int.  It handles addition and      //
// subtraction that make sure the number does not overflow or       //
// underflow.                                                       //
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>

#include <srcomm3/inout.h>
#include <srcomm3/longint.h>

static const long MAXLONG = 2000000000L;
static const long MINLONG =-2000000000L;

LongInteger::LongInteger( long nx )
{
	assign( nx );
}

void CC LongInteger::assign( long nx )
{
	if( nx > MAXLONG )
		x = MAXLONG;
	else if( nx < MINLONG )
		x = 0;
	else
		x = nx;
}

LongInteger& CC LongInteger::operator = ( long nx )
{
	assign( nx );
	return *this;
}

LongInteger& CC LongInteger::operator += ( long dx )
{
	if( dx < 0 ) return *this -= (-dx);

	long y = x;
	long dy = 0x10000000L;
	while( dy > 0 )
	{
		while( y < MAXLONG && dx >= dy && y+dy > y )
		{
			y += dy;
			dx -= dy;
		}
		dy /= 2;
	}

	assign( y );
	return *this;
}

LongInteger& CC LongInteger::operator -= ( long dx )
{
	if( dx < 0 ) return *this += (-dx);

	long y = x;
	long dy = 0x10000000L;
	while( dy > 0 )
	{
		while( y > MINLONG && dx >= dy && y-dy < y )
		{
			y -= dy;
			dx -= dy;
		}
		dy /= 2;
	}

	assign( y );
	return *this;
}

standardout& operator << ( standardout& out, const LongInteger& x )
{
	return out << (long)x;
}

int overflow( long x, long y )
{
	if( x >= 0 && y >= 0 && x+y < 0 ) return 1;
	if( x >= 0 && y >= 0 && x+y > MAXLONG ) return 1;
	if( x <= 0 && y <= 0 && x+y > 0 ) return 1;
	if( x <= 0 && y <= 0 && x+y < MINLONG ) return 1;
	return 0;
}
