#ifndef SRCOMM3_Menu_h
#define SRCOMM3_Menu_h

#include <unit/DynArray.h>
#include <srcomm3/action.h>

class MenuItem
{
  public:
    enum Position { LEFT, CENTER, RIGHT };

  private:
    enum { NumParams = 5 };

	string name;
	char key;
	int code;
	byte bc, kc;
	Array<byte,0,NumParams> c;

	int paramc;
	Array<string,0,NumParams> paramv;
	Array<Position,0,NumParams> paramp;

	// display the item
	void display( int lastcol, int showkeys, Array<int,0,NumParams>& w );
	// the length of the displayed line
	int width( int showkeys, Array<int,0,NumParams>& w );
	int paramwidth( int param );
	// update the colors
	void updatecolors();

	friend class Menu;

  public:
	MenuItem();
	MenuItem( string name, string description, char key, int cd, Position p = LEFT );
	~MenuItem();

	string Description()    { return paramv[0]; }

	void SetColors( byte b, byte k, byte x0, byte x1 = 7, byte x2 = 7, byte x3 = 7, byte x4 = 7, byte x5 = 7 );

	int AddParam( long l, Position p = RIGHT );
	int AddParam( const char *s, Position p = LEFT );
	int AddParam( const string& s, Position p = LEFT );
};

template<class T> CallbackAction<T>* MenuCallback( T* const obj, void ( T::*f )() )
{
	return new CallbackAction<T>( obj, f );
}

#ifndef __OS2__
template<class T> CallbackAction<T>* MenuCallback( T* obj, void ( T::*f )() )
{
	return new CallbackAction<T>( obj, f );
}
#endif

class Menu
{
  private:
	string name;
	string title;
	int maxcols;
	int interruptcode;
	Action* prompt;
	Action* header;
	Action* interrupthandler;
	byte tc, lc, bc, kc;
	Array<byte,0,MenuItem::NumParams> c;

    DynArray<MenuItem> item;

	friend class MenuItem;

  public:
	// Constructor:  (colors in order of occurrence in Display)
	//      string name
	//      string title
	//      byte title-color
	//      byte line-color
	//      byte bracket-color
	//      byte key-color
	//      byte description-color
	//      byte color-1 through color-5
	Menu( string, string, byte = 15, byte = 7, byte = 8, byte = 15, byte = 7, byte = 7, byte = 7, byte = 7, byte = 7, byte = 7 );
	~Menu();

	int Go( bool showmenu = true, int def = -1 );
	void Display();
    int DisplayPrompt();

	void SetInterrupt( int specialcode )	   { interruptcode = specialcode; }
	void SetInterrupt( Action* action )	   	   { interruptcode = -1; interrupthandler = action; }
	void SetCols( int cols = 1 )               { maxcols = cols; }
	int NumItems()                             { return item.length(); }

	int AddItem( int code, char k, string s, MenuItem::Position p = MenuItem::LEFT );
    MenuItem* Item( int i );

	void SetPrompt( Action* p )    { prompt = p; }
	void SetHeader( Action* p )    { header = p; }

	static char KeyBracketL;
	static char KeyBracketR;
	static char TitleBracketL;
	static char TitleBracketR;
	static char DefaultBracketL;
	static char DefaultBracketR;

	static char SecretKey;
	static char (*SecretHandler)();

	static int  Waiting;
	static int  Interrupted;

    static void StandardPrompt();
	static void StandardHeader();
};

#endif
